﻿// Copyright 2009-2011 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

// Using discriminated unions, a representation
// of a binary tree.

type 'a Tree = 
    | Empty
    | Leaf of 'a
    | Tree of 'a * 'a Tree * 'a Tree
    
let tree = 
    Tree(10, 
        Tree(5, 
            Leaf(3), 
            Leaf(7)
        ), 
        Tree(15,
            Tree(12,
                Leaf(11),
                Empty
            ),
            Leaf(20)
        )
    )

// a higher order function walkTree, which takes a
// tree and a function as its parameters, iterates over the tree
// and calls the function for each of the tree values. The iteration
// happens in order from the "leftmost" to the "rightmost" element.

let rec walkTree f t = 
    match t with 
    | Leaf(v) -> f t
    | Tree(v,l,r) ->
        walkTree f l
        f t
        walkTree f r
    | Empty -> f t
    
// Using the walkTree function to output the content of a test
// tree on the console.

let outputElement e = 
    match e with 
    | Empty -> printfn "Empty"
    | Leaf(v) -> printfn "Leaf(%A)" v
    | Tree(v,_,_) -> printfn "Element(%A)" v
    
walkTree outputElement tree