﻿// Copyright 2009-2011 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

open Helpers

// a discriminated union can look like an enum, but here
// it is actually compiled as a class
type MemberKind = 
    | Method
    | Property
    | Field
    
// these two are equal, as long as there's no conflict on the name Method
let mk = MemberKind.Method
let mk2 = Method

// in this case the compiler actually creates an enum
type MemberKind2 = 
    | Method = 1
    | Property = 2
    | Field = 3

// elements of discriminated unions can be more detailed
type Product = 
    | OwnProduct of string
    | RemoteReference of int
type Count = int
type StoreBooking = 
    | Incoming of Product * Count
    | Outgoing of Product * Count

let bookings = 
    [
        Incoming(OwnProduct("Rubber Hose"), 20);
        Incoming(RemoteReference(301), 100);
        Outgoing(RemoteReference(301), 40);
        Outgoing(OwnProduct("Boat"), 3)
    ]

// definitions can be recursive
type Caption = string
type Checked = bool
type StrContent = string
type Control = 
    | Button of Caption
    | CheckButton of Caption * Checked
    | Edit of Caption * StrContent
    | Container of Control list

let form = Container(
            [  
                Edit("User name", "<enter username here>");
                Edit("Password", "****");
                CheckButton("Remember password", true);
                Container(
                    [
                        Button("Ok");
                        Button("Cancel")
                    ])
            ])

open System

Output.banner "Tree of controls"

// this function iterates the tree of controls to
// produce a visual representation
let rec showControl c = 
    match c with 
    | Button(caption) -> 
        let sep = new String('-', caption.Length + 4)
        Console.WriteLine sep
        printfn "| %s |" caption
        Console.WriteLine sep
    | CheckButton(caption,isChecked) ->
        printfn "%s %s" (if isChecked then "X" else "O") caption
    | Edit(caption,content) ->
        printfn "%s: | %s |" caption (if content.Length = 0 then new String(' ', 5) else content)
    | Container(children) ->
        let sep = new String('=', 40)
        Console.WriteLine sep
        for control in children do
            showControl control
        Console.WriteLine sep

showControl form

Output.banner "Int List"

// an (incomplete) implementation of an int list, based
// on discriminated unions and tuples
type IntList = 
    | Empty
    | Element of int * IntList

let emptyIntList = Empty
let intList = Element(10, Element(20, Element(30, Empty)))
let intList2 = Element(40, Element(50, Empty))

let rec outputIntList l = 
    match l with 
    | Empty -> printfn "END"
    | Element(h, t) -> printfn "%A" h; outputIntList t
    
outputIntList intList

let consIntList element list = Element(element, list)
let rec appendIntList list1 list2 = 
    match list1 with
    | Empty -> list2
    | Element(h, t) when t = Empty -> 
        match list2 with
        | Empty -> list1
        | Element _ as e -> Element(h, e)
    | Element(h, t) -> Element(h, appendIntList t list2)

let appendedResult = appendIntList intList intList2
outputIntList appendedResult

let (++++) e l = consIntList e l
let (@@@@) l1 l2 = appendIntList l1 l2
    
let opConsResult = 10 ++++ intList2
let opAppendedResult = intList @@@@ intList2

outputIntList opAppendedResult
outputIntList opConsResult

